/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "r3matrix.h"

#include <QString>
#include <stdexcept>


namespace GerPh {

  R3Matrix::R3Matrix() {
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        fCoeff[i][j] = 0.0;
      } // for
    } // for
  } // R3Matrix::R3Matrix

  R3Matrix& R3Matrix::addMtx(const R3Matrix &M) {
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        fCoeff[i][j] += M.getCoeff(i, j);
      } // for
    } // for
    return *this;
  } // R3Matrix::addMtx

  void R3Matrix::checkIdx(int i, int j) {
    if (i < 0 || i >= 3 || j < 0 || j >= 3) {
      throw std::out_of_range(qPrintable(QString().sprintf(
        "Index [%i,%i] not within valid bounds [0..2,0..2]", i, j
      )));
    } // if
  } // R3Matrix::checkIdx

  double R3Matrix::getCoeff(int i, int j) const {
    checkIdx(i, j);
    return fCoeff[i][j];
  } // R3Matrix::getCoeff

  bool R3Matrix::isEqual(const R3Matrix &M) const {
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        if (fCoeff[i][j] != M.getCoeff(i, j)) {
	  return false;
	} // if
      } // for
    } // for
    return true;
  } // R3Matrix::isEqual

  R3Matrix& R3Matrix::multDbl(double x) {
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        fCoeff[i][j] *= x;
      } // for
    } // for
    return *this;
  } // R3Matrix::multDbl

  R3Matrix& R3Matrix::multMtx(const R3Matrix &M) {
    *this = *this * M;
    return *this;
  } // R3Matrix::multMtx

  void R3Matrix::setCoeff(int i, int j, double aValue) {
    checkIdx(i, j);
    fCoeff[i][j] = aValue;
  } // R3Matrix::setCoeff

  R3Matrix& R3Matrix::setEqual(const R3Matrix &M) {
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        fCoeff[i][j] = M.getCoeff(i, j);
      } // for
    } // for
    return *this;
  } // R3Matrix::setEqual

  void R3Matrix::setM(
    double a00, double a01, double a02,
    double a10, double a11, double a12,
    double a20, double a21, double a22
  ) {
    fCoeff[0][0] = a00;
    fCoeff[0][1] = a01;
    fCoeff[0][2] = a02;
    fCoeff[1][0] = a10;
    fCoeff[1][1] = a11;
    fCoeff[1][2] = a12;
    fCoeff[2][0] = a20;
    fCoeff[2][1] = a21;
    fCoeff[2][2] = a22;
  } // R3Matrix::setM

  R3Matrix& R3Matrix::subMtx(const R3Matrix &M) {
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        fCoeff[i][j] -= M.getCoeff(i, j);
      } // for
    } // for
    return *this;
  } // R3Matrix::subMtx

  R3Matrix R3Matrix::transpose() {
    R3Matrix result;
    int i, j;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        result.setCoeff(i, j, fCoeff[j][i]);
      } // for
    } // for
    return result;
  } // R3Matrix::transpose

  R3Matrix operator+ (const R3Matrix &M1, const R3Matrix &M2) {
    R3Matrix result = M1;
    result += M2;
    return result;
  } // operator+

  R3Matrix operator- (const R3Matrix &M1, const R3Matrix &M2) {
    R3Matrix result = M1;
    result -= M2;
    return result;
  } // operator-

  R3Matrix operator* (const R3Matrix &M1, const R3Matrix &M2) {
    R3Matrix result;
    int i, j, k;
    double x;
    for (i = 0; i < 3; i++) {
      for (j = 0; j < 3; j++) {
        x = 0.0;
	for (k = 0; k < 3; k++) {
	  x += M1.getCoeff(i, k) * M2.getCoeff(k, j);
	} // for
	result.setCoeff(i, j, x);
      } // for
    } // for
    return result;
  } // operator*

  R3Matrix operator* (double x, const R3Matrix &M) {
    R3Matrix result = M;
    result *= x;
    return result;
  } // operator-

  R3Matrix operator* (const R3Matrix &M, double x) {
    R3Matrix result = M;
    result *= x;
    return result;
  } // operator-

  R3Matrix operator/ (const R3Matrix &M, double x) {
    R3Matrix result = M;
    result /= x;
    return result;
  } // operator-

  R3Vector operator* (const R3Matrix &M, const R3Vector &v) {
    R3Vector result;
    int i, j;
    double x;
    for (i = 0; i < 3; i++) {
      x = 0.0;
      for (j = 0; j < 3; j++) {
        x += M.getCoeff(i, j) * v.getAt(j);
      } // for
      result.setAt(i, x);
    } // for
    return result;
  } // operator*

} // GerPh
