/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "r3vector.h"

#include <QString>
#include <math.h>
#include <stdexcept>


namespace GerPh {

  R3Vector::R3Vector() {
    int i;
    for (i = 0; i < 3; i++) {
      fCoeff[i] = 0.0;
    } // for
  } // R3Vector::R3Vector

  R3Vector& R3Vector::addVec(const R3Vector &v) {
    int i;
    for (i = 0; i < 3; i++) {
      fCoeff[i] += v.getAt(i);
    } // for
    return *this;
  } // R3Vector::addVec

  void R3Vector::checkIdx(int i) {
    if (i < 0 || i >= 3) {
      throw std::out_of_range(qPrintable(QString().sprintf(
        "i=%i: not within valid index range. [0..2]", i
      )));
    } // if
  } // R3Vector::checkIdx

  R3Vector& R3Vector::crossVec(const R3Vector &v) {
    *this = *this % v;
    return *this;
  } // R3Vector::crossVec

  double R3Vector::getAbs() {
    return sqrt((*this) * (*this));
  } // R3Vector::getAbs

  double R3Vector::getAt(int i) const {
    checkIdx(i);
    return fCoeff[i];
  } // R3Vector::getAt

  double R3Vector::getX() const {
    return fCoeff[0];
  } // R3Vector::getX

  double R3Vector::getY() const {
    return fCoeff[1];
  } // R3Vector::getY

  double R3Vector::getZ() const {
    return fCoeff[2];
  } // R3Vector::getZ

  bool R3Vector::isEqual(const R3Vector &v) const {
    int i;
    for (i = 0; i < 3; i++) {
      if (fCoeff[i] != v.getAt(i)) {
        return false;
      } // if
    } // for
    return true;
  } // R3Vector::isEqual

  R3Vector& R3Vector::multDbl(double x) {
    int i;
    for (i = 0; i < 3; i++) {
      fCoeff[i] *= x;
    } // for
    return *this;
  } // R3Vector::multDbl

  void R3Vector::setAt(int i, double aValue) {
    checkIdx(i);
    fCoeff[i] = aValue;
  } // R3Vector::setAt

  R3Vector& R3Vector::setEqual(const R3Vector &v) {
    int i;
    for (i = 0; i < 3; i++) {
      fCoeff[i] = v.getAt(i);
    } // for
    return *this;
  } // R3Vector::setEqual

  void R3Vector::setXYZ(double aX, double aY, double aZ) {
    fCoeff[0] = aX;
    fCoeff[1] = aY;
    fCoeff[2] = aZ;
  } // R3Vector::setXYZ

  R3Vector& R3Vector::subVec(const R3Vector &v) {
    int i;
    for (i = 0; i < 3; i++) {
      fCoeff[i] -= v.getAt(i);
    } // for
    return *this;
  } // R3Vector::subVec

  R3Vector operator+ (const R3Vector &v1, const R3Vector &v2) {
    R3Vector result = v1;
    result += v2;
    return result;
  } // operator+

  R3Vector operator- (const R3Vector &v1, const R3Vector &v2) {
    R3Vector result = v1;
    result -= v2;
    return result;
  } // operator-

  double operator* (const R3Vector &v1, const R3Vector &v2) {
    double result = 0.0;
    int i;
    for (i = 0; i < 3; i++) {
      result += v1.getAt(i) * v2.getAt(i);
    } // for
    return result;
  } // operator*

  R3Vector operator* (double x, const R3Vector &v) {
    R3Vector result = v;
    result *= x;
    return result;
  } // operator*

  R3Vector operator* (const R3Vector &v, double x) {
    R3Vector result = v;
    result *= x;
    return result;
  } // operator*

  R3Vector operator/ (const R3Vector &v, double x) {
    R3Vector result = v;
    result /= x;
    return result;
  } // operator/

  R3Vector operator% (const R3Vector &v1, const R3Vector &v2) {
    R3Vector result;
    double x1 = v1.getX();
    double y1 = v1.getY();
    double z1 = v1.getZ();
    double x2 = v2.getX();
    double y2 = v2.getY();
    double z2 = v2.getZ();
    result.setXYZ(
      y1 * z2 - z1 * y2,
      z1 * x2 - x1 * z2,
      x1 * y2 - y1 * x2
    );
    return result;
  } // operator%

} // GerPh
