/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "si.h"

#include <cmath>


namespace GerPh {

  const double SI::unit_nm	= 1.0e-9 * SI::unit_m;
  const double SI::unit_cm	= 1.0e-2 * SI::unit_m;
  const double SI::unit_m	= 1.0;
  const double SI::unit_km	= 1.0e+3 * SI::unit_m;
  const double SI::unit_ls	= SI::CONST_c * SI::unit_s;
  const double SI::unit_ly	= SI::CONST_c * SI::unit_year;

  const double SI::unit_m2	= SI::unit_m * SI::unit_m;
  const double SI::unit_cm2	= SI::unit_cm * SI::unit_cm;
  const double SI::unit_m3	= SI::unit_m2 * SI::unit_m;
  const double SI::unit_cm3	= SI::unit_cm2 * SI::unit_cm;

  const double SI::unit_g	= 1.0e-3 * SI::unit_kg;
  const double SI::unit_kg	= 1.0;
  const double SI::unit_t	= 1.0e+3 * SI::unit_kg;

  const double SI::unit_s	= 1.0;
  const double SI::unit_min	= 60.0 * SI::unit_s;
  const double SI::unit_h	= 60.0 * SI::unit_min;
  const double SI::unit_day	= 24.0 * SI::unit_h;
  const double SI::unit_year	= 365.25636574 * SI::unit_day;

  const double SI::unit_K	= 1.0;

  const double SI::unit_s2	= SI::unit_s * SI::unit_s;

  const double SI::unit_km_s	= SI::unit_km / SI::unit_s;

  const double SI::unit_g_cm3	= SI::unit_g / SI::unit_cm3;

  const double SI::unit_kg_s	= SI::unit_kg / SI::unit_s;

  const double SI::unit_N	= SI::unit_kg * SI::unit_m / SI::unit_s2;
  const double SI::unit_J	= SI::unit_N * SI::unit_m;
  const double SI::unit_W	= SI::unit_J / SI::unit_s;

  const double SI::unit_deg	= SI::CONST_pi / 180.0;
  const double SI::unit_arcmin	= SI::unit_deg / 60.0;
  const double SI::unit_arcsec	= SI::unit_arcmin / 60.0;

  const double SI::CONST_pi	= 3.1415926535897932384;
  const double SI::CONST_c	= 299792458.0 * SI::unit_m / SI::unit_s;
  const double SI::CONST_h	= 6.626e-34 * SI::unit_J * SI::unit_s;
  const double SI::CONST_G	= 6.6742867e-11 * SI::unit_m3 /
  				  (SI::unit_kg * SI::unit_s2);
  const double SI::CONST_g	= 9.81 * SI::unit_m / SI::unit_s2;
  const double SI::CONST_k	= 1.380662e-23 * SI::unit_J / SI::unit_K;

  double SI::ArcToRad(double aDeg, double aMin, double aSec) {
    double deg = aDeg;
    bool negative = aDeg < 0.0;
    if (negative) {
      deg = -aDeg;
    } // if
    double result = (((aSec / 60.0) + aMin) / 60.0 + deg) * CONST_pi / 180.0;
    if (negative) {
      result = -result;
    } // if
    return result;
  } // SI::ArcToRad

  double SI::EqToRad(double aH, double aMin, double aS) {
    return (((aS / 60.0) + aMin) / 60.0 + aH) * CONST_pi / 12.0;
  } // SI::EqToRad

  void SI::RadToArc(double aRad, double &aDeg, double &aMin, double &aSec) {
    double r = aRad * 180.0 / CONST_pi;
    aDeg = trunc(r);
    r = std::abs((r - aDeg) * 60.0);
    aMin = trunc(r);
    aSec = (r - aMin) * 60.0;
  } // SI::RadToArc

  void SI::RadToEq(double aRad, double &aH, double &aMin, double &aS) {
    double r = aRad * 12.0 / CONST_pi;
    while (r < 0.0) {
      r += 24.0;
    } // while
    while (r >= 24.0) {
      r -= 24.0;
    } // while
    aH = trunc(r);
    r = (r - aH) * 60.0;
    aMin = trunc(r);
    aS = (r - aMin) * 60.0;
  } // SI::RadToEq

} // GerPh
